















# [A](https://codeforces.com/gym/105143/problem/M)

给定由 $n$ 个数组成的序列 $\{a\}$，每次操作可以合并两个大小相差为 $1$ 的数，合并的结果是它们的和。

最终将剩下的数从大到小排序，求最大字典序。

$n\le 2\times 10^5,1\le a_i\le 10^{18}$

## Sol

- 每次将一个偶数和一个奇数合并成为一个奇数

由于题目要求最大字典序，所以按照贪心策略，每次要优先合并出来能够合并的最大的数。同时由于合并无法得到偶数，必须利用原有的偶数来进行合并。

假设当前最大的能够合并的偶数是 $x$，那么利用 $x$ 可以合并出来的数是 $2x-1$ 或 $2x+1$。

对于当前要得到一个数 $k$ 的过程：

- 如果当前序列中存在 $k$，则直接返回
- 如果不存在且 $k$ 为奇数，当且仅当可以得到 $k/2$ 和 $k-k/2$ 时才可以得到 $k$
- 如果不存在且 $k$ 为偶数，则不能得到 $k$

每次递归问题规模减小一半，利用哈希表或集合等可以在 $O(n\log a_i)$ 或 $O(n\log n\log a_i)$ 的复杂度内完成。

# [B](https://codeforces.com/gym/105143/problem/E)

给定一棵 $n$ 个点的树，根为 $r$，边权为 $1$，设 $V_1(t)$ 代表距离根节点不超过 $t$ 的点的集合。

给定 $t_0$，$V_2(r_0,k,t)$ 代表距离点 $r_0$ 不超过 $k(t-t_0)$ 的点的集合，规定 $t\ge t_0$。

对于每一个 $k\in[1,n]$，$r_0$ 可以任选，需要找到最小的 $t$ 使得 $V_1$ 被 $V_2$ 所包含。

$n\le 2\times 10^5,1\le r,t_0\le n$

## Sol

显然对于一个固定的 $t$，$r_0$ 选择在距离根节点不超过 $t$ 的子树的直径中点最优。

- 一棵树添加一个叶子后，直径最多改变一个端点。

对于当前直径 $(u,v)$，在加入 $x$ 后只需要判断 $(u,x),(v,x),(u,v)$ 之间的距离哪一个最大即可。节点之间的距离可以通过维护节点深度和 lca 求得。

当 $k$ 增大时 $t$ 为单调不增的趋势，对于所有可能的 $t$ 维护出子树直径，然后利用双指针可以 $O(n)$ 求得每一个 $k$ 的答案。

时间复杂度 $O(n\log n)$。

# [C](https://codeforces.com/gym/105143/problem/D)

给定长为 $n$ 的排成一排的格子，每个格子上面有一个整数 $a_i$。

定义一个参数为 $(s,t)$ 的操作为：初始化计数器为 $0$，从第 $s$ 个格子出发，每一步可以选择向左移动一步或向右移动一步，最多操作 $t$ 步。当第一次访问到某个格子（包括起点）时，将计数器加上格子上面的值。存在最优的操作方式可以使计数器的值最大，并且该操作的结果就是计数器的最大值。

对于所有的 $1\le s\le n,1\le t\le 2n$，请分别求出这些操作的结果。

$n\le 5000,0\le a_i\le 10^9$

## Sol

- 移动过程中最多改变一次方向，否则一定不优。

设 $g_{s,t}$ 代表从 $s$ 出发走不超过 $t$ 步，中途不改变方向的最大和。

对于一个不折返方向的操作，答案就是 $g_{s,t}$，反之，如果在 $x$ 折返，相当于从 $x$ 出发走 $t-|s-x|$ 步，即 $g_{x,t-|s-x|}$，注意此时 $g$ 的方向并不重要，因为 $a_i$ 非负，如果是同向必然不优。

所以操作 $(s,t)$ 的结果就是 $\max\{g_{s,t},g_{s+1,t-1},g_{s+2,t-2},...,g_{s-1,t-1},g_{s-2,t-2},...\}$

分别对于 $s+t$ 为定值的 $g$ 求一个后缀 $\max$，对 $s-t$ 为定值的 $g$ 求前缀 $\max$ 即可。

# [D](https://codeforces.com/gym/104633)

给定一棵 $n$ 个点的树，边有正边权，要找两条路径覆盖树上所有的边且路径总长度最小。

$n\le 10^5$

![image-20240502215917845](C:\Users\HP\AppData\Roaming\Typora\typora-user-images\image-20240502215917845.png)

## Sol

对于两条路径而言，如果存在重合的边，那么必然可以重新安排这两条路径使得总长度不变且两条路径不再重合。

这样相当于两条路径只会相交于一个点，且它们分别覆盖了这个点的所有子树里的边。

对于一棵树而言，用一条路径覆盖树的所有边的最优走法是选取最长链的两端作为起点和终点，链上的边只走一次，其余边走两次。

上面的问题相当于选取两条不在树边上相交的链使得总长度最长。

- 如果这两条链不相交，可以枚举边 $(u,v)$ 将整棵树分为两棵子树，分别取子树直径
- 如果相交，可以枚举交点 $u$，取从 $u$ 出发的四条最长的到叶子的链。

类似于 DP 求直径的方式，需要维护子树的直径和从每个点出发到叶子的最长链，利用换根 DP 可以解决。

时间复杂度 $O(n)$。

# [E](https://qoj.ac/contest/1516/problem/8237)

有 $n$ 个数 $a_1,a_2,...,a_n$，有 $n$ 个事件，第 $i$ 个事件给定 $b_i,w_i$，形如：如果 $a_i<a_{b_i}$，则 $a_i=a_i+w_i$。事件随机顺序发生，求每个数的期望。

$n\le 5\times 10^5,1\le b_i\le n,1\le w_i\le 10^9$

## Sol

事件的依赖关系构成若干棵基环树。

如果 $a_i< a_{b_i}$，则事件必然发生，如果 $a_i\ge a_{b_i}+w_{b_i}$，则事件必然不发生。

对于其它的事件，它发生的必要条件是前置条件在它之前发生。

对于基环树的环，如果环只有一个点，则该点的事件必然不可能发生。反之必然存在一条边满足 $a_i < a_{b_i}$。

故每个事件发生的概率就由离它最近的必然（不）发生的祖先节点决定，如果离它最近的祖先节点必然不发生，则它也必然不发生，反之设距离为 $L$，则其发生的概率为 $\frac{1}{L!}$。

直接 $O(n)$ dfs 一遍图即可。